// qaeng.cpp
//
#include <e32std.h>
#include <e32math.h>
#include <f32file.h>
#include <s32stor.h>
#include <s32file.h>

#include "qaeng.h"

EXPORT_C TInt E32Dll( TDllReason ) {
  return 0;
}

EXPORT_C CQAEng * CQAEng::NewL(TFileName& aQfile, TFileName& aAfile)
{
  CQAEng* self = new (ELeave) CQAEng;
  CleanupStack::PushL(self);
  self->ConstructL(aQfile, aAfile);
  CleanupStack::Pop();
  return self;
}

CQAEng::CQAEng()
{
	iUsed = EFalse;
	iQuestions = 0;
	iAnswers = 0;
}

// ConstructL
void CQAEng::ConstructL(TFileName& aQfile, TFileName& aAfile) {

  iQuestions = new (ELeave) CArrayPtrFlat<HBufC>( kMaxLen );
  iAnswers = new (ELeave) CArrayPtrFlat<HBufC>( kMaxLen );

  ReadWordsFromFileL(aQfile, iQuestions);
  ReadWordsFromFileL(aAfile, iAnswers);

  Reset();
}

void CQAEng::ReadWordsFromFileL(TFileName& aName, 
								CArrayPtrFlat<HBufC> * aData)
{
  // Tiedostonksittelyn muuttujat
  RFs fs;
  RFile file;
  // Yhteys tiedostopalvelimeen, ei voi leavata
  fs.Connect();

  // Tiedoston avaaminen, ei voi leavata
  file.Open( fs, aName, EFileShareReadersOnly );    

  // Sanojen luku, leavin sattuessa palataan thn
  TRAPD( error, ReadWordsL( file, aData ) );
  file.Close();
  fs.Close();

  if(error != KErrNone || (aData->Count() == 0)) {
    User::Leave( -1 ); // Mitn ei ole tehtviss, goodbye
  }
}

// ReadWordsL
// Luetaan annetun tiedoston kaikki rivit eli bingolapun
// termit sanataulukkoon
void CQAEng::ReadWordsL( RFile& aFile, CArrayPtrFlat<HBufC>* aData ) {
   // Lukeminen rivi kerrallaan. Rivin maksimipituus 256 merkki.
  TFileText ftxt;
  ftxt.Set( aFile );
  TBuf<256> buf;  

  // Luetaan kaikki sanat listaan
  while( ftxt.Read( buf ) == KErrNone || aData -> Count() == kMaxLen) {
    HBufC* term = buf.AllocL();
    CleanupStack::PushL( term );
    aData->AppendL( term );
	CleanupStack::Pop();  // term
  }
}

EXPORT_C CQAEng::~CQAEng() 
{
	if (iQuestions) {
		iQuestions->ResetAndDestroy();
		delete iQuestions;
	}
	if (iAnswers) {
		iAnswers->ResetAndDestroy();
		delete iAnswers;
	}
};


EXPORT_C void CQAEng::Select()
{
    iUsed = ETrue;
}

EXPORT_C const TDesC & CQAEng::Question() const
{
	return *iQuestion;
}
 
EXPORT_C const TDesC & CQAEng::Answer() const
{
	return *iAnswer;
}

EXPORT_C TBool CQAEng::Used() 
{
	return iUsed;
}

EXPORT_C void CQAEng::Reset()
{
  TInt i;
  TInt index;
  TInt64 ticks;

  ticks = User::TickCount();
  index = Math::Rand( ticks ) % iQuestions->Count();
  iQuestion = iQuestions->At(index);

  ticks = User::TickCount();
  index = Math::Rand( ticks ) % iAnswers->Count();
  iAnswer = iAnswers->At(index);
  iUsed = EFalse;
}

